#!/bin/bash
#   author:martinmhan@yahoo.com date:  22/04/2014
#   scadat is a bash command line interface to the tango scada
#   Copyright (C) <2014>  <Martin Mohan>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software Foundation,
#   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

#set -x
#verbose=0
#if [ x$1 = x"" ]; then 
#    export verbose=0
#else
#    export verbose=1
#fi

PASS=0
FAIL=1
export PATH=.:$PATH
#myscadat=`which scadat`
echo "`python --version` `which scadat`"

if [ -z "$TANGO_HOST" ]; then
    echo "set TANGO_HOST (e.g. export TANGO_HOST=127.0.0.1:10000)"
    exit 1
fi  

# Test bad TANGO_HOST's
TEMP=`echo $TANGO_HOST`
unset TANGO_HOST
assertgrep.sh "scadat -v" "=scadat HOST="
export TANGO_HOST=127.0.0.1:1234
assertequal.sh "scadat -l" "127.0.0.1:1234 openDB <class 'socket.error'>" stderr
export TANGO_HOST="InvalidHostName:1234"
assertequal.sh "scadat -l" "InvalidHostName:1234 openDB <class 'socket.gaierror'>" stderr
assertgrep.sh "scadat -v" "InvalidHostName:1234 openDB <class 'socket.gaierror'>" stderr
export TANGO_HOST=$TEMP

assertgrep.sh "scadat -v -t InvalidHostName:1234" "InvalidHostName:1234 openDB <class 'socket.gaierror'>" stderr

assertgrep.sh "scadat -v -t mytango:10000" "mytango:10000 openDB <class 'socket.gaierror'>" stderr

##exit 1
assertequal.sh "scadat" "scadat -h: For usage
scadat -v: For info"
assertgrep.sh "scadat -h" "Linux command line interface to tango"
assertgrep.sh "scadat -v" "scadat_version"
assertequal.sh "scadat -a bad/device/name" "Cannot read device bad/device/name" stderr
assertgrep.sh "scadat -a" "scadat: error: argument -a: expected at least one argument" stderr
assertequal.sh "scadat -a aa/bb/cc" "Cannot read device aa/bb/cc" stderr
assertequal.sh "scadat -a AABBCC" "stderr: Invalid name AABBCC (expected aa/bb/cc/<dd>)" stderr
assertequal.sh "scadat -a baddevicename" "stderr: Invalid name baddevicename (expected aa/bb/cc/<dd>)" stderr

assertgrep.sh "scadat -l -v" "sys/tg_test/1"
if (( $? == $FAIL )) ; then
    echo "$0: Further tests require running device sys/tg_test/1 at TANGO_HOST:$TANGO_HOST"
    exit 0
fi

assertgrep.sh "scadat -l" "sys/tg_test/1"
assertgrep.sh "scadat -c sys/tg_test/1" "sys/tg_test/1/Status"
assertgrep.sh "scadat -c sys/tg_test/1 -v" "sys/tg_test/1/Status <cmd>"
assertgrep.sh "scadat -c sys/tg_test/1 -vv" "sys/tg_test/1/Status <DevVoid>"
#assertgrep.sh "scadat -c sys/tg_test/1 -v" "CommandInfo"
assertequal.sh "scadat -c sys/tg_test/1/Init" "Usage:scadat -c sys/tg_test/1/Init DevVoid"
assertequal.sh "scadat -c sys/tg_test/1/Init DevVoid" "None"
assertequal.sh "scadat -a sys/tg_test/1 | grep Status" "sys/tg_test/1/Status"
assertequal.sh "scadat -a sys/tg_test/1 -v | grep Status" "sys/tg_test/1/Status <attr_ro>"
assertgrep.sh "scadat -c sys/tg_test/1/DevFloat 12.34" "12.34"
assertequal.sh "scadat -c sys/tg_test/1/Status" "Usage:scadat -c sys/tg_test/1/Status DevVoid"
assertequal.sh "scadat -c sys/tg_test/1/Status DevVoid" "The device is in RUNNING state."
assertequal.sh "scadat -c sys/tg_test/1/DevDouble 12.34" "12.34"
assertequal.sh "scadat -c sys/tg_test/1/DevLong 1234" "1234"
assertequal.sh "scadat -c sys/tg_test/1/DevLong64 1234" "1234"
assertequal.sh "scadat -c sys/tg_test/1/DevULong64 1234" "1234"
assertequal.sh "scadat -c sys/tg_test/1/DevULong 1234" "1234"
assertequal.sh "scadat -c sys/tg_test/1/DevBoolean 0" "False"
assertequal.sh "scadat -c sys/tg_test/1/DevBoolean 1" "True"
assertequal.sh "scadat -c sys/tg_test/1/DevShort 1234" "1234"
assertequal.sh "scadat -c sys/tg_test/1/DevUShort 1234" "1234"
assertequal.sh "scadat -c sys/tg_test/1/DevString 'my string'" "my string"
assertgrep.sh "scadat -c sys/tg_test/1/DevVarFloatArray '1.123456 2.2 3.3 4.4'" "1.123456"
assertgrep.sh "scadat -c sys/tg_test/1/DevVarDoubleArray '1.123456 2.2 3.3 4.4'" "1.123456"
#assertequal.sh "scadat -c sys/tg_test/1/DevVarUShortArray '1 2 3 4'" "[1, 2, 3, 4]"
#assertequal.sh "scadat -c sys/tg_test/1/DevVarShortArray '1 2 3 4'" "[1, 2, 3, 4]"
assertequal.sh "scadat -c sys/tg_test/1/DevVarStringArray 'string1 string2 string3 string4'" "['string1', 'string2', 'string3', 'string4']"
assertequal.sh "scadat -c sys/tg_test/1/DevVarShortArray 'string1 string2 string3 string4'" "Error writing cmd DevVarShortArray->string1 string2 string3 string4" stderr
assertequal.sh "scadat -c sys/tg_test/1/DevVoid DevVoid" "None"
assertgrep.sh "scadat -a sys/tg_test/1 | grep long_scalar_rww" "sys/tg_test/1/long_scalar_rww"
#assertequal.sh "scadat -a sys/tg_test/1 | grep ulong_image_ro" "sys/tg_test/1/ulong_image_ro <attr_ro>"
assertgrep.sh "scadat -a sys/tg_test/1 | grep 'short_scalar_rww'" "sys/tg_test/1/short_scalar_rww"
# Read 1 command
assertgrep.sh "scadat -a sys/tg_test/1 | grep ushort_scalar" "sys/tg_test/1/ushort_scalar"
assertgrep.sh "scadat -a sys/tg_test/1 | grep long_scalar_w" "sys/tg_test/1/long_scalar_w"
# Write/Read Attributes
assertgrep.sh "scadat -a sys/tg_test/1" "sys/tg_test/1"
assertequal.sh "scadat -a sys/tg_test/1 123" "Usage: scadat -a aa/bb/cc/dd 123" stderr
assertequal.sh "scadat -a sys/tg_test/1 123" "Usage: scadat -a aa/bb/cc/dd 123" stderr
assertequal.sh "scadat -a sys/tg_test/1/long_scalar_w" "sys/tg_test/1/long_scalar_w"
assertequal.sh "scadat -a sys/tg_test/1/long_scalar_w -v" "sys/tg_test/1/long_scalar_w <attr_w>"
assertequal.sh "scadat -a sys/tg_test/1/long_scalar_w 4321" "sys/tg_test/1/long_scalar_w->4321"
assertequal.sh "scadat -a sys/tg_test/1/long_scalar_w -vv" "sys/tg_test/1/long_scalar_w 4321"
assertequal.sh "scadat -a sys/tg_test/1/long_scalar_w 1234" "sys/tg_test/1/long_scalar_w->1234"
assertequal.sh "scadat -a sys/tg_test/1/long_scalar_w -vv" "sys/tg_test/1/long_scalar_w 1234"
assertequal.sh "scadat -a sys/tg_test/1/long_scalar_w" "sys/tg_test/1/long_scalar_w"
assertequal.sh "scadat -a sys/tg_test/1/long_scalar_w -v" "sys/tg_test/1/long_scalar_w <attr_w>"
assertequal.sh "scadat -a sys/tg_test/1/string_scalar 'my string 123'" "sys/tg_test/1/string_scalar->my string 123"
assertequal.sh "scadat -a sys/tg_test/1/string_scalar" "sys/tg_test/1/string_scalar"
assertequal.sh "scadat -a sys/tg_test/1/string_scalar -v" "sys/tg_test/1/string_scalar <attr_w>"
assertgrep.sh "scadat -a sys/tg_test/1/string_scalar -vv" "sys/tg_test/1/string_scalar my string 123"
assertgrep.sh "scadat -a bad/device/name sys/tg_test/1" "sys/tg_test/1" stderr
# Fix bug devices printed twice
assertequal.sh "scadat -l -vv | grep dserver/DataBaseds | wc -l" "1" 
    assertequal.sh "scadat -c sys/tg_test/1/DevLong" "Usage:scadat -c sys/tg_test/1/DevLong DevLong" stderr
assertgrep.sh "scadat -c sys/tg_test/1/badcommand" "reason = API_CommandNotFound" stderr
#assertgrep.sh "scadat_search.sh 198.23.242.110 2" "nc -nvz -w1 198.23.242.111 10000 - closed"
assertgrep.sh "scadat -a sys/tg_test/1" "sys/tg_test/1/Status"

assertequal.sh "scadat -p sys/tg_test/1" ""
assertequal.sh "scadat -p sys/tg_test/1 123" "Usage: scadat -p aa/bb/cc/prop 123" stderr
assertequal.sh "scadat -r sys/tg_test/1/Status" "Usage: scadat -r sys/tg_test/1" stderr

assertgrep.sh "scadat -l" "test/simulator/rampx2"
if (( $? == $FAIL )) ; then
    echo "$0: Further tests require running device test/simulator/rampx2 at TANGO_HOST:$TANGO_HOST"
    exit 0
fi
# Testing props need test/simulator/rampx2
assertgrep.sh "scadat -p test/simulator/rampx2/*" "test/simulator/rampx2/DeviceNr"
assertgrep.sh "scadat -p test/simulator/rampx2" "test/simulator/rampx2/DeviceNr"
assertgrep.sh "scadat -p test/simulator/rampx2/DeviceNr -v" "test/simulator/rampx2/DeviceNr <prop>"
assertgrep.sh "scadat -p test/simulator/rampx2/DeviceNr -vv" "test/simulator/rampx2/DeviceNr: DeviceNr ["
assertgrep.sh "scadat -p test/simulator/rampx2/DeviceNr 2" "test/simulator/rampx2/DeviceNr ['2']->2"
# Second write (I know current value is 2)
assertequal.sh "scadat -p test/simulator/rampx2/DeviceNr 2" "test/simulator/rampx2/DeviceNr ['2']->2"
assertequal.sh "scadat -c sys/tg_test/1/DevVarLongArray '1 2 3 4'" "[1 2 3 4]"
assertequal.sh "scadat -c sys/tg_test/1/DevVarLong64Array '1 2 3 4'" "[1 2 3 4]"
assertequal.sh "scadat -c sys/tg_test/1/DevVarULong64Array '1 2 3 4'" "[1 2 3 4]"
assertequal.sh "scadat -c sys/tg_test/1/DevVarUShortArray '1 2 3 4'" "[1 2 3 4]"
assertequal.sh "scadat -c sys/tg_test/1/DevVarShortArray '1 2 3 4'" "[1 2 3 4]"
assertequal.sh "scadat -a sys/tg_test/1 | grep ulong_image_ro" "sys/tg_test/1/ulong_image_ro"

#assertgrep.sh "scadat badhostname:1000" "scan_port:Error reading badhostname:1000" stderr
#export TANGO_HOST=scadat.org:10000
#result=$?
##echo "result is $result"
#if [ "$result" != "0" ]; then
#    echo "Check network: nc -vz -w1 scadat.org 10000 - Network access- failed"
#else
#    echo "Check network: nc -vz -w1 scadat.org 10000 - Network access- ok"
#fi

#assertequal.sh "scadat -a sys/tg_test/1 | grep short_scalar_w" "sys/tg_test/1/short_scalar_w 4321"
#assertequal.sh "scadat -a sys/tg_test/1 short_scalar_w 4321" "short_scalar_w->4321"
#### Not supported - yet
#assertgrep.sh "$scadat -a sys/tg_test/1 DevVarDoubleStringArray '1 2 3 4'" "1 2 3 4"
#assertgrep.sh "$scadat -a sys/tg_test/1 DevVarCharArray '1 2 3 4'" "1 2 3 4"
#assertgrep.sh "$scadat -a sys/tg_test/1 DevVarLongStringArray 1234" "1234"
